@@ -6,7 +6,7 @@ a person from the IMDb database.
 It can fetch data through different media (e.g.: the IMDb web pages,
 a SQL database, etc.)
 
-Copyright 2004-2010 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2011 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 __all__ = ['IMDb', 'IMDbError', 'Movie', 'Person', 'Character', 'Company',
             'available_access_systems']
-__version__ = VERSION = '4.5'
+__version__ = VERSION = '4.8'
 
 # Import compatibility module (importing it is enough).
 import _compat
@@ -38,6 +38,8 @@ import imdb._logging
 from imdb._exceptions import IMDbError, IMDbDataAccessError
 from imdb.utils import build_title, build_name, build_company_name
 
+_aux_logger = logging.getLogger('imdbpy.aux')
+
 
 # URLs of the main pages for movies, persons, characters and queries.
 imdbURL_base = 'http://akas.imdb.com/'
@@ -98,8 +100,7 @@ class ConfigParserWithCase(ConfigParser.ConfigParser):
                 self.read(fname)
             except (ConfigParser.MissingSectionHeaderError,
                     ConfigParser.ParsingError), e:
-                import warnings
-                warnings.warn('Troubles reading config file: %s' % e)
+                _aux_logger.warn('Troubles reading config file: %s' % e)
             # Stop at the first valid file.
             if self.has_section('imdbpy'):
                 break
@@ -155,9 +156,8 @@ def IMDb(accessSystem=None, *arguments, **keywords):
             kwds.update(keywords)
             keywords = kwds
         except Exception, e:
-            import warnings
-            warnings.warn('Unable to read configuration file; ' + \
-                            'complete error: %s' % e)
+            logging.getLogger('imdbpy').warn('Unable to read configuration' \
+                                            ' file; complete error: %s' % e)
             # It just LOOKS LIKE a bad habit: we tried to read config
             # options from some files, but something is gone horribly
             # wrong: ignore everything and pretend we were called with
@@ -173,29 +173,29 @@ def IMDb(accessSystem=None, *arguments, **keywords):
             import logging.config
             logging.config.fileConfig(os.path.expanduser(logCfg))
         except Exception, e:
-            import warnings
-            warnings.warn('unable to read logger config: %s' % e)
+            logging.getLogger('imdbpy').warn('unable to read logger ' \
+                                            'config: %s' % e)
+    if accessSystem in ('httpThin', 'webThin', 'htmlThin'):
+        logging.warn('httpThin was removed since IMDbPY 4.8')
+        accessSystem = 'http'
     if accessSystem in ('http', 'web', 'html'):
         from parser.http import IMDbHTTPAccessSystem
         return IMDbHTTPAccessSystem(*arguments, **keywords)
-    elif accessSystem in ('httpThin', 'webThin', 'htmlThin'):
-        from parser.http import IMDbHTTPAccessSystem
-        return IMDbHTTPAccessSystem(isThin=1, *arguments, **keywords)
     elif accessSystem in ('mobile',):
         from parser.mobile import IMDbMobileAccessSystem
         return IMDbMobileAccessSystem(*arguments, **keywords)
     elif accessSystem in ('local', 'files'):
         # The local access system was removed since IMDbPY 4.2.
-        raise IMDbError, 'the local access system was removed since IMDbPY 4.2'
+        raise IMDbError('the local access system was removed since IMDbPY 4.2')
     elif accessSystem in ('sql', 'db', 'database'):
         try:
             from parser.sql import IMDbSqlAccessSystem
         except ImportError:
-            raise IMDbError, 'the sql access system is not installed'
+            raise IMDbError('the sql access system is not installed')
         return IMDbSqlAccessSystem(*arguments, **keywords)
     else:
-        raise IMDbError, 'unknown kind of data access system: "%s"' \
-                            % accessSystem
+        raise IMDbError('unknown kind of data access system: "%s"' \
+                            % accessSystem)
 
 
 def available_access_systems():
@@ -204,7 +204,7 @@ def available_access_systems():
     # XXX: trying to import modules is a good thing?
     try:
         from parser.http import IMDbHTTPAccessSystem
-        asList += ['http', 'httpThin']
+        asList.append('http')
     except ImportError:
         pass
     try:
@@ -362,7 +362,7 @@ class IMDbBase:
         """Return a list of tuples (movieID, {movieData})"""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def search_movie(self, title, results=None, _episodes=False):
         """Return a list of Movie objects for a query for the given title.
@@ -389,7 +389,7 @@ class IMDbBase:
         """Return a list of tuples (movieID, {movieData})"""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def search_episode(self, title, results=None):
         """Return a list of Movie objects for a query for the given title.
@@ -423,7 +423,7 @@ class IMDbBase:
         """Return a list of tuples (personID, {personData})"""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def search_person(self, name, results=None):
         """Return a list of Person objects for a query for the given name.
@@ -468,7 +468,7 @@ class IMDbBase:
         """Return a list of tuples (characterID, {characterData})"""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def search_character(self, name, results=None):
         """Return a list of Character objects for a query for the given name.
@@ -513,7 +513,7 @@ class IMDbBase:
         """Return a list of tuples (companyID, {companyData})"""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def search_company(self, name, results=None):
         """Return a list of Company objects for a query for the given name.
@@ -536,7 +536,7 @@ class IMDbBase:
         """Return a list of 'keyword' strings."""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def search_keyword(self, keyword, results=None):
         """Search for existing keywords, similar to the given one."""
@@ -554,7 +554,7 @@ class IMDbBase:
         """Return a list of tuples (movieID, {movieData})"""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def get_keyword(self, keyword, results=None):
         """Return a list of movies for the given keyword."""
@@ -579,7 +579,7 @@ class IMDbBase:
         #      subclass, somewhere under the imdb.parser package.
         #      This method must return a list of (movieID, {movieDict})
         #      tuples.  The kind parameter can be 'top' or 'bottom'.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def get_top250_movies(self):
         """Return the list of the top 250 movies."""
@@ -672,16 +672,16 @@ class IMDbBase:
             mopID = mop.companyID
             prefix = 'company'
         else:
-            raise IMDbError, 'object ' + repr(mop) + \
-                    ' is not a Movie, Person, Character or Company instance'
+            raise IMDbError('object ' + repr(mop) + \
+                    ' is not a Movie, Person, Character or Company instance')
         if mopID is None:
             # XXX: enough?  It's obvious that there are Characters
             #      objects without characterID, so I think they should
             #      just do nothing, when an i.update(character) is tried.
             if prefix == 'character':
                 return
-            raise IMDbDataAccessError, \
-                'the supplied object has null movieID, personID or companyID'
+            raise IMDbDataAccessError( \
+                'the supplied object has null movieID, personID or companyID')
         if mop.accessSystem == self.accessSystem:
             aSystem = self
         else:
@@ -705,11 +705,14 @@ class IMDbBase:
                 continue
             if not i:
                 continue
+            self._imdb_logger.debug('retrieving "%s" info set', i)
             try:
                 method = getattr(aSystem, 'get_%s_%s' %
                                     (prefix, i.replace(' ', '_')))
             except AttributeError:
-                raise IMDbDataAccessError, 'unknown information set "%s"' % i
+                self._imdb_logger.error('unknown information set "%s"', i)
+                # Keeps going.
+                method = lambda *x: {}
             try:
                 ret = method(mopID)
             except Exception, e:
@@ -741,28 +744,28 @@ class IMDbBase:
         web server); must be overridden by the subclass."""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def get_imdbPersonID(self, personID):
         """Translate a personID in a imdbID (the ID used by the IMDb
         web server); must be overridden by the subclass."""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def get_imdbCharacterID(self, characterID):
         """Translate a characterID in a imdbID (the ID used by the IMDb
         web server); must be overridden by the subclass."""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def get_imdbCompanyID(self, companyID):
         """Translate a companyID in a imdbID (the ID used by the IMDb
         web server); must be overridden by the subclass."""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
-        raise NotImplementedError, 'override this method'
+        raise NotImplementedError('override this method')
 
     def _searchIMDb(self, kind, ton):
         """Search the IMDb akas server for the given title or name."""
@@ -854,8 +857,8 @@ class IMDbBase:
             else:
                 imdbID = aSystem.company2imdbID(build_company_name(mop))
         else:
-            raise IMDbError, 'object ' + repr(mop) + \
-                        ' is not a Movie, Person or Character instance'
+            raise IMDbError('object ' + repr(mop) + \
+                        ' is not a Movie, Person or Character instance')
         return imdbID
 
     def get_imdbURL(self, mop):
@@ -873,8 +876,8 @@ class IMDbBase:
         elif isinstance(mop, Company.Company):
             url_firstPart = imdbURL_company_main
         else:
-            raise IMDbError, 'object ' + repr(mop) + \
-                        ' is not a Movie, Person, Character or Company instance'
+            raise IMDbError('object ' + repr(mop) + \
+                        ' is not a Movie, Person, Character or Company instance')
         return url_firstPart % imdbID
 
     def get_special_methods(self):
