@@ -4,7 +4,7 @@ parser.http.utils module (imdb package).
 This module provides miscellaneous utilities used by
 the imdb.parser.http classes.
 
-Copyright 2004-2009 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2011 Davide Alberani <da@erlug.linux.it>
                2008 H. Turgut Uyar <uyar@tekir.org>
 
 This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
 import re
+import logging
 import warnings
 
 from imdb._exceptions import IMDbError
@@ -88,7 +89,7 @@ entitydefs = entitydefs.copy()
 entitydefsget = entitydefs.get
 entitydefs['nbsp'] = ' '
 
-sgmlentity = {'lt': '<', 'gt': '>', 'amp': '&', 'quot': '"', 'apos': '\''}
+sgmlentity = {'lt': '<', 'gt': '>', 'amp': '&', 'quot': '"', 'apos': '\'', 'ndash': '-'}
 sgmlentityget = sgmlentity.get
 _sgmlentkeys = sgmlentity.keys()
 
@@ -171,10 +172,13 @@ def subSGMLRefs(s):
     return re_sgmlrefsub(_replSGMLRefs, s)
 
 
+_b_p_logger = logging.getLogger('imdbpy.parser.http.build_person')
 def build_person(txt, personID=None, billingPos=None,
                 roleID=None, accessSystem='http', modFunct=None):
     """Return a Person instance from the tipical <tr>...</tr> strings
     found in the IMDb's web site."""
+    #if personID is None
+    #    _b_p_logger.debug('empty name or personID for "%s"', txt)
     notes = u''
     role = u''
     # Search the (optional) separator between name and role/notes.
@@ -241,10 +245,14 @@ def build_person(txt, personID=None, billingPos=None,
         if lr == 1:
             role = role[0]
             roleID = roleID[0]
+            notes = roleNotes[0] or u''
     elif roleID is not None:
         roleID = str(roleID)
     if personID is not None:
         personID = str(personID)
+    if (not name) or (personID is None):
+        # Set to 'debug', since build_person is expected to receive some crap.
+        _b_p_logger.debug('empty name or personID for "%s"', txt)
     # XXX: return None if something strange is detected?
     person = Person(name=name, personID=personID, currentRole=role,
                     roleID=roleID, notes=notes, billingPos=billingPos,
@@ -256,13 +264,20 @@ def build_person(txt, personID=None, billingPos=None,
     return person
 
 
+_re_chrIDs = re.compile('[0-9]{7}')
+
+_b_m_logger = logging.getLogger('imdbpy.parser.http.build_movie')
 # To shrink spaces.
 re_spaces = re.compile(r'\s+')
 def build_movie(txt, movieID=None, roleID=None, status=None,
                 accessSystem='http', modFunct=None, _parsingCharacter=False,
-                _parsingCompany=False):
+                _parsingCompany=False, year=None, chrRoles=None,
+                rolesNoChar=None, additionalNotes=None):
     """Given a string as normally seen on the "categorized" page of
     a person on the IMDb's web site, returns a Movie instance."""
+    # FIXME: Oook, lets face it: build_movie and build_person are now
+    # two horrible sets of patches to support the new IMDb design.  They
+    # must be rewritten from scratch.
     if _parsingCharacter:
         _defSep = ' Played by '
     elif _parsingCompany:
@@ -280,10 +295,20 @@ def build_movie(txt, movieID=None, roleID=None, status=None,
         role = tsplit[1].lstrip()
     if title[-9:] == 'TV Series':
         title = title[:-9].rstrip()
+    #elif title[-7:] == '(short)':
+    #    title = title[:-7].rstrip()
+    #elif title[-11:] == '(TV series)':
+    #    title = title[:-11].rstrip()
+    #elif title[-10:] == '(TV movie)':
+    #    title = title[:-10].rstrip()
     elif title[-14:] == 'TV mini-series':
         title = title[:-14] + ' (mini)'
+    if title and title.endswith(_defSep.rstrip()):
+        title = title[:-len(_defSep)+1]
     # Try to understand where the movie title ends.
     while True:
+        if year:
+            break
         if title[-1:] != ')':
             # Ignore the silly "TV Series" notice.
             if title[-9:] == 'TV Series':
@@ -305,19 +330,32 @@ def build_movie(txt, movieID=None, roleID=None, status=None,
         if (first4.isdigit() or first4 == '????') and \
                 title[nidx+5:nidx+6] in (')', '/'): break
         # The last item in parentheses is a known kind: stop here.
-        if title[nidx+1:-1] in ('TV', 'V', 'mini', 'VG'): break
+        if title[nidx+1:-1] in ('TV', 'V', 'mini', 'VG', 'TV movie',
+                'TV series', 'short'): break
         # Else, in parentheses there are some notes.
         # XXX: should the notes in the role half be kept separated
         #      from the notes in the movie title half?
         if notes: notes = '%s %s' % (title[nidx:], notes)
         else: notes = title[nidx:]
         title = title[:nidx].rstrip()
+    if year:
+        year = year.strip()
+        if title[-1] == ')':
+            fpIdx = title.rfind('(')
+            if fpIdx != -1:
+                if notes: notes = '%s %s' % (title[fpIdx:], notes)
+                else: notes = title[fpIdx:]
+                title = title[:fpIdx].rstrip()
+        title = u'%s (%s)' % (title, year)
     if _parsingCharacter and roleID and not role:
         roleID = None
     if not roleID:
         roleID = None
     elif len(roleID) == 1:
         roleID = roleID[0]
+    if not role and chrRoles and isinstance(roleID, (str, unicode)):
+        roleID = _re_chrIDs.findall(roleID)
+        role = ' / '.join(filter(None, chrRoles.split('@@')))
     # Manages multiple roleIDs.
     if isinstance(roleID, list):
         tmprole = role.split('/')
@@ -346,13 +384,35 @@ def build_movie(txt, movieID=None, roleID=None, status=None,
         roleID = str(roleID)
     if movieID is not None:
         movieID = str(movieID)
+    if (not title) or (movieID is None):
+        _b_m_logger.error('empty title or movieID for "%s"', txt)
+    if rolesNoChar:
+        rolesNoChar = filter(None, [x.strip() for x in rolesNoChar.split('/')])
+        if not role:
+            role = []
+        elif not isinstance(role, list):
+            role = [role]
+        role += rolesNoChar
+    notes = notes.strip()
+    if additionalNotes:
+        additionalNotes = re_spaces.sub(' ', additionalNotes).strip()
+        if notes:
+            notes += u' '
+        notes += additionalNotes
+    if role and isinstance(role, list) and notes.endswith(role[-1].replace('\n', ' ')):
+        role = role[:-1]
+    if isinstance(role, list): print 'R', [r.encode('ascii', 'ignore') for r in role]
+    else: role.encode('ascii', 'ignore')
     m = Movie(title=title, movieID=movieID, notes=notes, currentRole=role,
                 roleID=roleID, roleIsPerson=_parsingCharacter,
                 modFunct=modFunct, accessSystem=accessSystem)
     if roleNotes and len(roleNotes) == len(roleID):
         for idx, role in enumerate(m.currentRole):
-            if roleNotes[idx]:
-                role.notes = roleNotes[idx]
+            try:
+                if roleNotes[idx]:
+                    role.notes = roleNotes[idx]
+            except IndexError:
+                break
     # Status can't be checked here, and must be detected by the parser.
     if status:
         m['status'] = status
@@ -368,6 +428,8 @@ class DOMParserBase(object):
     extractors = []
     usingModule = None
 
+    _logger = logging.getLogger('imdbpy.parser.http.domparser')
+
     def __init__(self, useModule=None):
         """Initialize the parser. useModule can be used to force it
         to use 'BeautifulSoup' or 'lxml'; by default, it's auto-detected,
@@ -395,28 +457,29 @@ class DOMParserBase(object):
                     self._is_xml_unicode = True
                     self.usingModule = 'beautifulsoup'
                 else:
-                    warnings.warn('unknown module "%s".' % mod)
+                    self._logger.warn('unknown module "%s"' % mod)
                     continue
                 self.fromstring = fromstring
                 self._tostring = tostring
                 if _gotError:
-                    warnings.warn('falling back to "%s".' % mod)
+                    warnings.warn('falling back to "%s"' % mod)
                 break
             except ImportError, e:
                 if idx+1 >= nrMods:
                     # Raise the exception, if we don't have any more
                     # options to try.
-                    raise IMDbError, 'unable to use any parser in %s: %s' % (
-                                                    str(useModule), str(e))
+                    raise IMDbError('unable to use any parser in %s: %s' % \
+                                    (str(useModule), str(e)))
                 else:
                     warnings.warn('unable to use "%s": %s' % (mod, str(e)))
                     _gotError = True
                 continue
         else:
-            raise IMDbError, 'unable to use parsers in %s' % str(useModule)
+            raise IMDbError('unable to use parsers in %s' % str(useModule))
         # Fall-back defaults.
         self._modFunct = None
         self._as = 'http'
+        self._cname = self.__class__.__name__
         self._init()
         self.reset()
 
@@ -452,47 +515,86 @@ class DOMParserBase(object):
         # Temporary fix: self.parse_dom must work even for empty strings.
         html_string = self.preprocess_string(html_string)
         html_string = html_string.strip()
-        # tag attributes like title="&#x22;Family Guy&#x22;" will be
-        # converted to title=""Family Guy"" and this confuses BeautifulSoup.
         if self.usingModule == 'beautifulsoup':
+            # tag attributes like title="&#x22;Family Guy&#x22;" will be
+            # converted to title=""Family Guy"" and this confuses BeautifulSoup.
             html_string = html_string.replace('""', '"')
+            # Browser-specific escapes create problems to BeautifulSoup.
+            html_string = html_string.replace('<!--[if IE]>', '"')
+            html_string = html_string.replace('<![endif]-->', '"')
+        #print html_string.encode('utf8')
         if html_string:
             dom = self.get_dom(html_string)
-            dom = self.preprocess_dom(dom)
+            #print self.tostring(dom).encode('utf8')
+            try:
+                dom = self.preprocess_dom(dom)
+            except Exception, e:
+                self._logger.error('%s: caught exception preprocessing DOM',
+                                    self._cname, exc_info=True)
             if self.getRefs:
-                self.gather_refs(dom)
+                try:
+                    self.gather_refs(dom)
+                except Exception, e:
+                    self._logger.warn('%s: unable to gather refs: %s',
+                                    self._cname, exc_info=True)
             data = self.parse_dom(dom)
         else:
             data = {}
-        data = self.postprocess_data(data)
+        try:
+            data = self.postprocess_data(data)
+        except Exception, e:
+            self._logger.error('%s: caught exception postprocessing data',
+                                self._cname, exc_info=True)
         if self._containsObjects:
             self.set_objects_params(data)
         data = self.add_refs(data)
         return data
 
+    def _build_empty_dom(self):
+        from bsouplxml import _bsoup
+        return _bsoup.BeautifulSoup('')
+
     def get_dom(self, html_string):
         """Return a dom object, from the given string."""
-        return self.fromstring(html_string)
+        try:
+            dom = self.fromstring(html_string)
+            if dom is None:
+                dom = self._build_empty_dom()
+                self._logger.error('%s: using a fake empty DOM', self._cname)
+            return dom
+        except Exception, e:
+            self._logger.error('%s: caught exception parsing DOM',
+                                self._cname, exc_info=True)
+            return self._build_empty_dom()
 
     def xpath(self, element, path):
         """Return elements matching the given XPath."""
-        xpath_result = element.xpath(path)
-        if self._is_xml_unicode:
-            return xpath_result
-
-        result = []
-        for item in xpath_result:
-            if isinstance(item, str):
-                item = unicode(item)
-            result.append(item)
-        return result
+        try:
+            xpath_result = element.xpath(path)
+            if self._is_xml_unicode:
+                return xpath_result
+            result = []
+            for item in xpath_result:
+                if isinstance(item, str):
+                    item = unicode(item)
+                result.append(item)
+            return result
+        except Exception, e:
+            self._logger.error('%s: caught exception extracting XPath "%s"',
+                                self._cname, path, exc_info=True)
+            return []
 
     def tostring(self, element):
         """Convert the element to a string."""
         if isinstance(element, (unicode, str)):
             return unicode(element)
         else:
-            return self._tostring(element, encoding=unicode)
+            try:
+                return self._tostring(element, encoding=unicode)
+            except Exception, e:
+                self._logger.error('%s: unable to convert to string',
+                                    self._cname, exc_info=True)
+                return u''
 
     def clone(self, element):
         """Clone an element."""
@@ -502,6 +604,8 @@ class DOMParserBase(object):
         """Here we can modify the text, before it's parsed."""
         if not html_string:
             return html_string
+        # Remove silly &nbsp;&raquo; chars.
+        html_string = html_string.replace(u' \xbb', u'')
         try:
             preprocessors = self.preprocessors
         except AttributeError:
@@ -513,7 +617,12 @@ class DOMParserBase(object):
             elif isinstance(src, str):
                 html_string = html_string.replace(src, sub)
             elif callable(src):
-                html_string = src(html_string)
+                try:
+                    html_string = src(html_string)
+                except Exception, e:
+                    _msg = '%s: caught exception preprocessing html'
+                    self._logger.error(_msg, self._cname, exc_info=True)
+                    continue
         ##print html_string.encode('utf8')
         return html_string
 
@@ -556,7 +665,12 @@ class DOMParserBase(object):
                     normalizer = extractor.group_key_normalize
                     if normalizer is not None:
                         if callable(normalizer):
-                            group_key = normalizer(group_key)
+                            try:
+                                group_key = normalizer(group_key)
+                            except Exception, e:
+                                _m = '%s: unable to apply group_key normalizer'
+                                self._logger.error(_m, self._cname,
+                                                    exc_info=True)
                     group_elements = self.xpath(group, extractor.path)
                     elements.extend([(group_key, element)
                                      for element in group_elements])
@@ -570,6 +684,7 @@ class DOMParserBase(object):
                             if not value:
                                 data[field] = None
                             else:
+                                # XXX: use u'' , to join?
                                 data[field] = ''.join(value)
                     else:
                         data = self.xpath(element, attr.path)
@@ -581,13 +696,21 @@ class DOMParserBase(object):
                         continue
                     attr_postprocess = attr.postprocess
                     if callable(attr_postprocess):
-                        data = attr_postprocess(data)
+                        try:
+                            data = attr_postprocess(data)
+                        except Exception, e:
+                            _m = '%s: unable to apply attr postprocess'
+                            self._logger.error(_m, self._cname, exc_info=True)
                     key = attr.key
                     if key is None:
                         key = group_key
                     elif key.startswith('.'):
                         # assuming this is an xpath
-                        key = self.xpath(element, key)[0]
+                        try:
+                            key = self.xpath(element, key)[0]
+                        except IndexError:
+                            self._logger.error('%s: XPath returned no items',
+                                                self._cname, exc_info=True)
                     elif key.startswith('self.'):
                         key = getattr(self, key[5:])
                     if attr.multi:
@@ -705,7 +828,7 @@ class GatherRefs(DOMParserBase):
                             'link': './@href',
                             'info': './following::text()[1]'
                             },
-        postprocess=lambda x: _parse_ref(x.get('text'), x.get('link'),
+        postprocess=lambda x: _parse_ref(x.get('text') or u'', x.get('link') or '',
                                          (x.get('info') or u'').strip()))]
     extractors = [
         Extractor(label='names refs',
@@ -728,6 +851,10 @@ class GatherRefs(DOMParserBase):
         for item in ('names refs', 'titles refs', 'characters refs'):
             result[item] = {}
             for k, v in data.get(item, []):
+                k = k.strip()
+                v = v.strip()
+                if not (k and v):
+                    continue
                 if not v.endswith('/'): continue
                 imdbID = analyze_imdbid(v)
                 if item == 'names refs':
